@@ -1,4 +1,6 @@
 # frozen_string_literal: true
+require 'resolv'
+
 module Excon
   class Socket
     include Utils
@@ -9,20 +11,27 @@ module Excon
 
     # read/write drawn from https://github.com/ruby-amqp/bunny/commit/75d9dd79551b31a5dd3d1254c537bad471f108cf
     CONNECT_RETRY_EXCEPTION_CLASSES = if defined?(IO::EINPROGRESSWaitWritable) # Ruby >= 2.1
-      [Errno::EINPROGRESS, IO::EINPROGRESSWaitWritable]
+      [Errno::EINPROGRESS, IO::EINPROGRESSWaitWritable].freeze
     else # Ruby <= 2.0
-      [Errno::EINPROGRESS]
+      [Errno::EINPROGRESS].freeze
     end
     READ_RETRY_EXCEPTION_CLASSES = if defined?(IO::EAGAINWaitReadable) # Ruby >= 2.1
-      [Errno::EAGAIN, Errno::EWOULDBLOCK, IO::WaitReadable, IO::EAGAINWaitReadable, IO::EWOULDBLOCKWaitReadable]
+      [Errno::EAGAIN, Errno::EWOULDBLOCK, IO::WaitReadable, IO::EAGAINWaitReadable, IO::EWOULDBLOCKWaitReadable].freeze
     else # Ruby <= 2.0
-      [Errno::EAGAIN, Errno::EWOULDBLOCK, IO::WaitReadable]
+      [Errno::EAGAIN, Errno::EWOULDBLOCK, IO::WaitReadable].freeze
     end
     WRITE_RETRY_EXCEPTION_CLASSES = if defined?(IO::EAGAINWaitWritable) # Ruby >= 2.1
-      [Errno::EAGAIN, Errno::EWOULDBLOCK, IO::WaitWritable, IO::EAGAINWaitWritable, IO::EWOULDBLOCKWaitWritable]
+      [Errno::EAGAIN, Errno::EWOULDBLOCK, IO::WaitWritable, IO::EAGAINWaitWritable, IO::EWOULDBLOCKWaitWritable].freeze
     else # Ruby <= 2.0
-      [Errno::EAGAIN, Errno::EWOULDBLOCK, IO::WaitWritable]
+      [Errno::EAGAIN, Errno::EWOULDBLOCK, IO::WaitWritable].freeze
     end
+    # Maps a socket operation to a timeout property.
+    OPERATION_TO_TIMEOUT = {
+      :connect_read => :connect_timeout,
+      :connect_write => :connect_timeout,
+      :read => :read_timeout,
+      :write => :write_timeout
+    }.freeze
 
     def params
       Excon.display_warning('Excon::Socket#params is deprecated use Excon::Socket#data instead.')
@@ -38,18 +47,22 @@ module Excon
 
     def_delegators(:@socket, :close)
 
+
     def initialize(data = {})
       @data = data
       @nonblock = data[:nonblock]
       @port ||= @data[:port] || 80
       @read_buffer = String.new
+      @read_offset = 0
       @eof = false
+      @backend_eof = false
+
       connect
     end
 
     def read(max_length = nil)
       if @eof
-        return max_length ? nil : ''
+        max_length ? nil : ''
       elsif @nonblock
         read_nonblock(max_length)
       else
@@ -58,19 +71,33 @@ module Excon
     end
 
     def readline
-      return legacy_readline if RUBY_VERSION.to_f <= 1.8_7
-      buffer = String.new
-      buffer << read_nonblock(1) while buffer[-1] != "\n"
-      buffer
-    end
+      if @nonblock
+        result = String.new
+        block = consume_read_buffer
 
-    def legacy_readline
-      begin
-        Timeout.timeout(@data[:read_timeout]) do
-          @socket.readline
+        loop do
+          idx = block.index("\n")
+
+          if idx.nil?
+            result << block
+          else
+            result << block[0..idx]
+            rewind_read_buffer(block, idx)
+            break
+          end
+
+          block = read_nonblock(@data[:chunk_size]) || raise(EOFError)
+        end
+
+        result
+      else # nonblock/legacy
+        begin
+          Timeout.timeout(@data[:read_timeout]) do
+            @socket.readline
+          end
+        rescue Timeout::Error
+          raise Excon::Errors::Timeout.new('read timeout reached')
         end
-      rescue Timeout::Error
-        raise Excon::Errors::Timeout.new('read timeout reached')
       end
     end
 
@@ -95,20 +122,27 @@ module Excon
     def connect
       @socket = nil
       exception = nil
+      hostname = @data[:hostname]
+      port = @port
+      family = @data[:family]
 
       if @data[:proxy]
-        family = @data[:proxy][:family] || ::Socket::Constants::AF_UNSPEC
-        args = [@data[:proxy][:hostname], @data[:proxy][:port], family, ::Socket::Constants::SOCK_STREAM]
-      else
-        family = @data[:family] || ::Socket::Constants::AF_UNSPEC
-        args = [@data[:hostname], @port, family, ::Socket::Constants::SOCK_STREAM]
+        hostname = @data[:proxy][:hostname]
+        port = @data[:proxy][:port]
+        family = @data[:proxy][:family]
       end
-      if RUBY_VERSION >= '1.9.2' && defined?(RUBY_ENGINE) && RUBY_ENGINE == 'ruby'
-        args << nil << nil << false # no reverse lookup
-      end
-      addrinfo = ::Socket.getaddrinfo(*args)
 
-      addrinfo.each do |_, port, _, ip, a_family, s_type|
+      resolver = @data[:resolv_resolver] || Resolv.new
+
+      # Deprecated
+      if @data[:dns_timeouts]
+        Excon.display_warning('dns_timeouts is deprecated, use resolv_resolver instead.')
+        dns_resolver = Resolv::DNS.new
+        dns_resolver.timeouts = @data[:dns_timeouts]
+        resolver = Resolv.new([Resolv::Hosts.new, dns_resolver])
+      end
+
+      resolver.each_address(hostname) do |ip|
         # already succeeded on previous addrinfo
         if @socket
           break
@@ -120,8 +154,8 @@ module Excon
         # nonblocking connect
         begin
           sockaddr = ::Socket.sockaddr_in(port, ip)
-
-          socket = ::Socket.new(a_family, s_type, 0)
+          addrinfo = Addrinfo.getaddrinfo(ip, port, family, :STREAM).first
+          socket = ::Socket.new(addrinfo.pfamily, addrinfo.socktype, addrinfo.protocol)
 
           if @data[:reuseaddr]
             socket.setsockopt(::Socket::Constants::SOL_SOCKET, ::Socket::Constants::SO_REUSEADDR, true)
@@ -147,10 +181,12 @@ module Excon
             socket.close rescue nil
           end
         rescue SystemCallError => exception
-          socket.close rescue nil if socket
+          socket&.close rescue nil
         end
       end
 
+      exception ||= Resolv::ResolvError.new("no address for #{hostname}")
+
       # this will be our last encountered exception
       fail exception unless @socket
 
@@ -172,20 +208,55 @@ module Excon
       end
     end
 
+    # Consume any bytes remaining in the read buffer before making a system call.
+    def consume_read_buffer
+      block = @read_buffer[@read_offset..-1]
+
+      @read_offset = @read_buffer.length
+
+      block
+    end
+
+    # Rewind the read buffer to just after the given index.
+    # The offset is moved back to the start of the current chunk and then forward until just after the index.
+    def rewind_read_buffer(chunk, idx)
+      @read_offset = @read_offset - chunk.length + (idx + 1)
+      @eof = false
+    end
+
     def read_nonblock(max_length)
       begin
+        if @read_offset != 0 && @read_offset >= @read_buffer.length
+          # Clear the buffer so we can test for emptiness below
+          @read_buffer.clear
+          # Reset the offset so it matches the length of the buffer when empty.
+          @read_offset = 0
+        end
+
         if max_length
-          until @read_buffer.length >= max_length
-            @read_buffer << @socket.read_nonblock(max_length - @read_buffer.length)
+          until @backend_eof || readable_bytes >= max_length
+            if @read_buffer.empty?
+              # Avoid allocating a new buffer string when the read buffer is empty
+              @read_buffer = @socket.read_nonblock(max_length, @read_buffer)
+            else
+              @read_buffer << @socket.read_nonblock(max_length - readable_bytes)
+            end
           end
         else
-          loop do
-            @read_buffer << @socket.read_nonblock(@data[:chunk_size])
+          until @backend_eof
+            if @read_buffer.empty?
+              # Avoid allocating a new buffer string when the read buffer is empty
+              @read_buffer = @socket.read_nonblock(@data[:chunk_size], @read_buffer)
+            else
+              @read_buffer << @socket.read_nonblock(@data[:chunk_size])
+            end
           end
         end
       rescue OpenSSL::SSL::SSLError => error
         if error.message == 'read would block'
-          select_with_timeout(@socket, :read) && retry
+          if @read_buffer.empty?
+            select_with_timeout(@socket, :read) && retry
+          end
         else
           raise(error)
         end
@@ -195,21 +266,37 @@ module Excon
           select_with_timeout(@socket, :read) && retry
         end
       rescue EOFError
-        @eof = true
+        @backend_eof = true
       end
 
       if max_length
         if @read_buffer.empty?
-          nil # EOF met at beginning
+          # EOF met at beginning
+          @eof = @backend_eof
+          nil
         else
-          @read_buffer.slice!(0, max_length)
+          start = @read_offset
+
+          # Ensure that we can seek backwards when reading until a terminator string.
+          # The read offset must never point past the end of the read buffer.
+          @read_offset += max_length > readable_bytes ? readable_bytes : max_length
+          @read_buffer[start...@read_offset]
         end
       else
         # read until EOFError, so return everything
-        @read_buffer.slice!(0, @read_buffer.length)
+        start = @read_offset
+
+        @read_offset = @read_buffer.length
+        @eof = @backend_eof
+
+        @read_buffer[start..-1]
       end
     end
 
+    def readable_bytes
+      @read_buffer.length - @read_offset
+    end
+
     def read_block(max_length)
       @socket.read(max_length)
     rescue OpenSSL::SSL::SSLError => error
@@ -219,9 +306,7 @@ module Excon
         raise(error)
       end
     rescue *READ_RETRY_EXCEPTION_CLASSES
-      if @read_buffer.empty?
-        select_with_timeout(@socket, :read) && retry
-      end
+      select_with_timeout(@socket, :read) && retry
     rescue EOFError
       @eof = true
     end
@@ -272,17 +357,33 @@ module Excon
     end
 
     def select_with_timeout(socket, type)
+      timeout_kind = type
+      timeout = @data[OPERATION_TO_TIMEOUT[type]]
+
+      # Check whether the request has a timeout configured.
+      if @data.include?(:deadline)
+        request_timeout = request_time_remaining
+
+        # If the time remaining until the request times out is less than the timeout for the type of select,
+        # use the time remaining as the timeout instead.
+        if request_timeout < timeout
+          timeout_kind = :request
+          timeout = request_timeout
+        end
+      end
+
       select = case type
       when :connect_read
-        IO.select([socket], nil, nil, @data[:connect_timeout])
+        IO.select([socket], nil, nil, timeout)
       when :connect_write
-        IO.select(nil, [socket], nil, @data[:connect_timeout])
+        IO.select(nil, [socket], nil, timeout)
       when :read
-        IO.select([socket], nil, nil, @data[:read_timeout])
+        IO.select([socket], nil, nil, timeout)
       when :write
-        IO.select(nil, [socket], nil, @data[:write_timeout])
+        IO.select(nil, [socket], nil, timeout)
       end
-      select || raise(Excon::Errors::Timeout.new("#{type} timeout reached"))
+
+      select || raise(Excon::Errors::Timeout.new("#{timeout_kind} timeout reached"))
     end
 
     def unpacked_sockaddr
@@ -292,5 +393,16 @@ module Excon
         raise
       end
     end
+
+    # Returns the remaining time in seconds until we reach the deadline for the request timeout.
+    # Raises an exception if we have exceeded the request timeout's deadline.
+    def request_time_remaining
+      now = Process.clock_gettime(Process::CLOCK_MONOTONIC)
+      deadline = @data[:deadline]
+
+      raise(Excon::Errors::Timeout.new('request timeout reached')) if now >= deadline
+
+      deadline - now
+    end
   end
 end
