@@ -3,13 +3,13 @@ require 'spec_helper'
 describe Excon::Socket do
   let(:dns_resolver) { Resolv::DNS.new }
   let(:config_timeouts) { dns_resolver.instance_variable_get(:@config).instance_variable_get(:@timeouts) }
-  let(:connection) { Excon.new('http://foo.com', dns_timeouts: 1) }
+  let(:connection) { Excon.new('http://google.com', dns_timeouts: 1) }
 
   before { allow(Resolv::DNS).to receive(:new).and_return(dns_resolver) }
 
   it 'passes the dns_timeouts to Resolv::DNS::Config' do
     silence_warnings do
-      connection.request
+      connection.connect
 
       expect(config_timeouts).to eql([1])
     end
