@@ -52,6 +52,49 @@ Shindo.tests('Excon stubs') do
     Excon.stubs.clear
   end
 
+  tests("stub({ url: 'http://example.com' }, { body: 'body', status: 200 })") do
+    Excon.stub({ url: 'http://example.com' }, { body: 'body', status: 200 })
+
+    tests('request without slash matches stub without trailing slash') do
+      response = Excon.get('http://example.com', mock: true)
+      response.status == 200 && response.body == 'body'
+    end
+
+    tests('request with trailing slash matches stub without trailing slash') do
+      response = Excon.get('http://example.com/', mock: true)
+      response.status == 200 && response.body == 'body'
+    end
+  end
+
+  tests("stub({ url: 'http://example.com?foo=bar' }, { body: 'body', status: 200 })") do
+    Excon.stub({ url: 'http://example.com?foo=bar' }, { body: 'body', status: 200 })
+
+    tests('query in url matches') do
+      response = Excon.get('http://example.com?foo=bar', mock: true)
+      response.status == 200 && response.body == 'body'
+    end
+
+    tests('hash query matches') do
+      response = Excon.get('http://example.com', query: { 'foo' => 'bar' }, mock: true)
+      response.status == 200 && response.body == 'body'
+    end
+  end
+
+  tests("stub({ url: 'http://example.com?a=b+c' }, { body: 'body', status: 200 })") do
+    Excon.stub({ url: 'http://example.com?a=b+c' }, { body: 'body', status: 200 })
+
+    tests('query in url matches') do
+      response = Excon.get('http://example.com?a=b+c', mock: true)
+      response.status == 200 && response.body == 'body'
+    end
+
+    tests('hash query matches') do
+      response = Excon.get('http://example.com', query: { 'a' => 'b c' }, mock: true)
+      response.status == 200 && response.body == 'body'
+    end
+  end
+
+
   tests("stub({:path => %r{/tests/(\S+)}}, {:body => $1, :status => 200})") do
     connection = nil
     response = nil
@@ -94,7 +137,12 @@ Shindo.tests('Excon stubs') do
       response.body
     end
 
-    tests('response.headers').returns({'Accept' => '*/*', 'Host' => '127.0.0.1:9292', 'User-Agent' => "excon/#{Excon::VERSION}"}) do
+    tests('response.headers').returns({
+                                        'Accept' => '*/*',
+                                        'Accept-Encoding' => 'deflate, gzip',
+                                        'Host' => '127.0.0.1:9292',
+                                        'User-Agent' => "excon/#{Excon::VERSION}"
+                                      }) do
       response.headers
     end
 
@@ -301,8 +349,8 @@ Shindo.tests('Excon stubs') do
   end
 
   tests("thread-local stubs") do
-    original_stubs_value = Excon.defaults[:stubs]
-    Excon.defaults[:stubs] = :local
+    original_defaults = Excon.defaults
+    Excon.defaults = Excon.defaults.merge(stubs: :local)
 
     connection = Excon.new('http://127.0.0.1:9292', :mock => true)
     Excon.stub({}, {:body => '1'})
@@ -318,7 +366,7 @@ Shindo.tests('Excon stubs') do
     end
     Excon.stubs.clear
 
-    Excon.defaults[:stubs] = original_stubs_value
+    Excon.defaults = original_defaults
   end
 
   env_restore
